package com.example.kpprinter.java;

import android.Manifest;
import android.annotation.SuppressLint;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothClass;
import android.bluetooth.BluetoothClass.Service;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.location.LocationManager;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.support.v4.content.ContextCompat;

import com.lvrenyang.io.BTPrinting;
import com.lvrenyang.io.IOCallBack;
import com.lvrenyang.io.Pos;
import com.printer.sdk.PrinterConstants;
import com.printer.sdk.PrinterInstance;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static android.content.pm.PackageManager.PERMISSION_GRANTED;

public class AndroidPrintSdk implements IOCallBack {
    private static AndroidPrintSdk printExamSdk;
    private Context context;
    private BluetoothAdapter blueAdapter;
    private BroadcastReceiver broadcastReceiver = null;
    private IntentFilter intentFilter = null;
    private LinkedHashMap<String, Object> printContentMap;//打印内容
    private String printerName = "";
    private String printerName2 = "";

    private BluetoothDevice mDevice;
    //MTP打印机
    private ExecutorService es = Executors.newScheduledThreadPool(30);
    private Pos mPos = new Pos();
    private BTPrinting mBt = new BTPrinting();
    //ID321打印机
    private static PrinterInstance myPrinter;
    private static boolean isConnected = false;// 蓝牙连接状态
    private static boolean isMTPConnected = false;// 蓝牙连接状态
    private boolean isChangePrint = false;
    private boolean isChangePrinter = false;//切换了打印机

    private Map<String, String> printMap;

    private PrintDeviceMsgListener printDeviceMsgListener;//蓝牙设备信息回调接口

    private BlueToothEnableListener blueToothEnableListner;//蓝牙是否打开的接口

    private DeviceConnectListener deviceConnectListener;//设备是否链接接口

    private PrintResultListener printResultListener;//打印结果接口

    private DeviceSearchListener deviceSearchListener;//设备搜索接口

    private LocationPermissionListener locationPermissionListener;//位置权限

    private List<Map<String, String>> connectAndPrintMsg;
    private String title, msg1, msg2, msgBtn, msgBtn2;
    ;
    private boolean isConnectAndPrint = false;
    private int blueCount = 0;

    public AndroidPrintSdk(Context context) {
        this.context = context.getApplicationContext();
    }

    public static AndroidPrintSdk getInstance(Context context) {
        if (printExamSdk == null) {
            synchronized (AndroidPrintSdk.class) {
                if (printExamSdk == null) {
                    printExamSdk = new AndroidPrintSdk(context);
                }
            }
        }
        return printExamSdk;
    }


    /**
     * 位置权限
     */
    public interface LocationPermissionListener {
        void close();
    }

    public void setOnLocationPermissionListener(LocationPermissionListener listener) {
        this.locationPermissionListener = listener;
    }

    /**
     * 蓝牙没有开启回调
     */
    public interface BlueToothEnableListener {
        void enableFalse();
    }

    public void setOnBlueToothEnableListener(BlueToothEnableListener listener) {
        this.blueToothEnableListner = listener;
    }

    /**
     * 设备连接结果回调
     */
    public interface DeviceConnectListener {
        void connectFalse();

        void connectSuccess();

        void connectUnConnect();
    }

    public void setOnDeviceConnectListener(DeviceConnectListener listener) {
        this.deviceConnectListener = listener;
    }

    /**
     * 打印结果回调
     */
    public interface PrintResultListener {
        void printSuccess();

        void printFail();
    }

    public void setOnPrintResultListener(PrintResultListener listener) {
        this.printResultListener = listener;
    }

    /**
     * 设备搜索接口
     */
    public interface DeviceSearchListener {
        void searchSuccess();

        void searchFail();
    }

    public void setOnDeviceSearchListener(DeviceSearchListener listener) {
        this.deviceSearchListener = listener;
    }

    /**
     * 打印机名字地址回调方法
     *
     * @param listener 打印机内容回调接口
     */
    public void setPrintDeviceMsgListener(PrintDeviceMsgListener listener) {
        this.printDeviceMsgListener = listener;
    }

    /**
     * 打印机内容回调接口
     */
    public interface PrintDeviceMsgListener {
        void printerMsg(LinkedHashMap<String, String> map);
    }

    /**
     * 有打印内容的情况
     *
     * @param printName 打印单子的名称
     * @param printMap  打印内容
     */
    public void initPrint(String printName, LinkedHashMap<String, Object> printMap) {
        this.printerName = printName;
        this.printContentMap = printMap;
        blueAdapter = BluetoothAdapter.getDefaultAdapter();
        mPos.Set(mBt);
        mBt.SetCallBack(this);
        initBroadcast();
    }

    /**
     * 初始化打印机
     */
    public void initPrint() {
        blueAdapter = BluetoothAdapter.getDefaultAdapter();
        mPos.Set(mBt);
        mBt.SetCallBack(this);
        initBroadcast();
    }

    /**
     * 搜索打印机
     */
    public void searchPrint(List<Map<String, String>> map) {

        title = map.get(1).get("title");
        msg1 = map.get(2).get("msg1");
        msg2 = map.get(3).get("msg2");
        msgBtn = map.get(4).get("msgBtn");
        msgBtn2 = map.get(5).get("msgBtn2");


        if (!blueAdapter.isEnabled())
            blueToothEnableListner.enableFalse();

        if (!showGPSContacts() || !blueAdapter.isEnabled()) {
            if (deviceSearchListener != null)
                deviceSearchListener.searchFail();
        }

        PermissionUtils.getLocations(title, msg1, msg2, msgBtn, msgBtn2, new PermissionUtils.PermissionTypeListener() {
            @Override
            public void success(Object msg) {
                if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
                    if (!isOPen(context)) {
                        if (locationPermissionListener != null)
                            locationPermissionListener.close();
//                            getAppDetailSettingIntent(context);
                    }
//
                    else {
                        blueAdapter.cancelDiscovery();
                        blueCount = 0;
                        blueAdapter.startDiscovery();
                    }
                } else {
                    blueAdapter.cancelDiscovery();
                    blueCount = 0;
                    blueAdapter.startDiscovery();
                }
            }

            @Override
            public void fail(String msg) {
//                    locationPermissionListener.close();

            }
        });
    }


    /**
     * 扫描蓝牙
     */
    private void initBroadcast() {
        broadcastReceiver = new BroadcastReceiver() {

            @Override
            public void onReceive(Context context, Intent intent) {

                String action = intent.getAction();
                Log.e("链接状态", intent.toString());
                mDevice = intent
                        .getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);

                if (BluetoothDevice.ACTION_FOUND.equals(action)) {
                    if (mDevice == null)
                        return;
                    final String address = mDevice.getAddress();
                    String name = mDevice.getName();
                    if (name == null)
                        name = "--";
                    else if (name.equals(address))
                        name = "--";


                    BluetoothClass btClass = mDevice.getBluetoothClass();
                    int nClass = 0;
                    if (btClass.hasService(Service.AUDIO))
                        nClass |= Service.AUDIO;
                    else if (btClass.hasService(Service.CAPTURE))
                        nClass |= Service.CAPTURE;
                    else if (btClass.hasService(Service.INFORMATION))
                        nClass |= Service.INFORMATION;
                    else if (btClass.hasService(Service.LIMITED_DISCOVERABILITY))
                        nClass |= Service.LIMITED_DISCOVERABILITY;
                    else if (btClass.hasService(Service.NETWORKING))
                        nClass |= Service.NETWORKING;
                    else if (btClass.hasService(Service.OBJECT_TRANSFER))
                        nClass |= Service.OBJECT_TRANSFER;
                    else if (btClass.hasService(Service.POSITIONING))
                        nClass |= Service.POSITIONING;
                    else if (btClass.hasService(Service.RENDER))
                        nClass |= Service.RENDER;
                    else if (btClass.hasService(Service.TELEPHONY))
                        nClass |= Service.TELEPHONY;

                    nClass |= btClass.getDeviceClass();

//                    String strClass = String.format("%06X", nClass);
//                    BlueBean blueBean = new BlueBean();
//                    blueBean.setName(name + ": " + address + "(" + strClass + ")");
//                    blueBean.setAddress(address);
////
//                    if(device.getBondState() == BluetoothDevice.BOND_BONDED ) {
//                        if(TextUtils.equals(blueBean.getAddress(), SPUtils.get(CpPrinterActivity.this, Constants.BLUETOOTH_ADDRESS,"").toString())){
//                            blueBean.setBonding(1);
//                        }else{
//                            blueBean.setBonding(-1);
//                        }
//
//                    }else{
//                        blueBean.setBonding(-1);
//                    }
//                    for (int i = 0; i < list.size(); ++i) {
//                        if(blueBean.getName().equals(list.get(i).getName()))
//                            return;
//                    }
//                    list.add(blueBean);
                    //todo sdk向外传蓝牙设备名称和地址
                    if (name.contains("MTP") || name.contains("printer") || name.contains("Printer")) {
                        LinkedHashMap<String, String> map = new LinkedHashMap<>();
                        map.put("name", name);
                        map.put("id", address);
                        if (printDeviceMsgListener != null)
                            printDeviceMsgListener.printerMsg(map);
                    }

                } else if (BluetoothAdapter.ACTION_DISCOVERY_STARTED
                        .equals(action)) {
                    //todo 显示loading UI
                    blueCount++;
                } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED
                        .equals(action)) {
                    if (deviceSearchListener != null && blueCount >= 1) {
                        deviceSearchListener.searchSuccess();
                        blueCount = 0;
                    }

                }

            }

        };
        intentFilter = new IntentFilter();
        intentFilter.addAction(BluetoothDevice.ACTION_FOUND);
        intentFilter.addAction(BluetoothAdapter.ACTION_DISCOVERY_STARTED);
        intentFilter.addAction(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        context.registerReceiver(broadcastReceiver, intentFilter);
    }

    /**
     * 断开打印机连接的时候注销广播
     */
    private void uninitBroadcast() {
        if (broadcastReceiver != null)
            context.unregisterReceiver(broadcastReceiver);
    }

    /**
     * 页面销毁需要调用的方法
     */
    public void finishView() {
        disPrinter(true);
        uninitBroadcast();
    }


    /**
     * 公共的连接打印机
     *
     * @param mapPrint
     */
    public void connectPrint(Map<String, String> mapPrint) {
        if (!blueAdapter.isEnabled()) {
            if (blueToothEnableListner != null)
                blueToothEnableListner.enableFalse();
            return;
        }
        isChangePrint = true;
        printMap = mapPrint;
        if (myPrinter != null)
            myPrinter.closeConnection();

        es.submit(new TaskClose(mBt));

        // MTP 打印机
        if (mapPrint.get("name").contains("MTP")) {
            Log.i("打印机address", mapPrint.get("id"));
            es.submit(new TaskOpen(mBt, printMap.get("id"), context));
        } else {
            //ID321打印连接
            connectID321(mapPrint.get("id"));
        }

    }


    public void printerToPrint(List<Map<String, String>> msg) {
//        if (!isMTPConnected && !isConnected) {
//            if (deviceConnectListener != null)
//                deviceConnectListener.connectUnConnect();
//            return;
//        }
        if (isMTPConnected)
            MTPPrint(msg);
        if (isConnected)
            IDPrint(msg);
//        if (!isMTPConnected && !isConnected)
//            if (deviceConnectListener != null)
//                deviceConnectListener.connectUnConnect();
    }


    /**
     * 断开打印机方式
     */
    public void disPrinter(boolean isDisView) {
        isChangePrint = false;
//        if (!isMTPConnected && !isConnected) {
//            if (deviceConnectListener != null)
//                deviceConnectListener.connectUnConnect();
//            return;
//        }
        if (isMTPConnected)
            disMTPPrinter();
        if (isConnected)
            disID321Printer();
        if (!isDisView)
            if (!isMTPConnected && !isConnected) {
                if (deviceConnectListener != null)
                    deviceConnectListener.connectFalse();
            }
    }


    /**
     * MTP 打印
     */
    private void MTPPrint(List<Map<String, String>> printMsg) {
        if (!blueAdapter.isEnabled()) {
            if (blueToothEnableListner != null)
                blueToothEnableListner.enableFalse();
            return;
        }

        es.submit(new TaskPrint(mPos, printMsg));
    }


    /**
     * MTP 打印机连接成功回调
     */
    @Override
    public void OnOpen() {


//        Toast.makeText(context,"MTP连接成功",Toast.LENGTH_SHORT).show();
        if (deviceConnectListener != null)
            deviceConnectListener.connectSuccess();
        isMTPConnected = true;
        if (isConnectAndPrint) {
            printerToPrint(msg);
            Log.i("MTP==", "MTP连接成功");
        }


    }

    /**
     * MTP 打印机连接失败回调
     */
    @Override
    public void OnOpenFailed() {

        if (deviceConnectListener != null && !isChangePrinter && !isMTPConnected)
            deviceConnectListener.connectFalse();
//        if (isChangePrint)
//            es.submit(new TaskOpen(mBt, printMap.get("id"), context));
        isMTPConnected = false;
    }

    /**
     * MTP 关闭打印机回调
     */
    @Override
    public void OnClose() {

        if (isChangePrint)
            if (deviceConnectListener != null && !isChangePrinter && !isMTPConnected)
                deviceConnectListener.connectFalse();
//        if (isChangePrint)
//            es.submit(new TaskOpen(mBt, printMap.get("id"), context));
        isMTPConnected = false;
    }

    /**
     * MTP 连接
     */
    public class TaskOpen implements Runnable {
        BTPrinting bt;
        String address;
        Context context;

        private TaskOpen(BTPrinting bt, String address, Context context) {
            this.bt = bt;
            this.address = address;
            this.context = context;
        }

        @Override
        public void run() {
            // TODO Auto-generated method stub
            bt.Open(address, context);
        }
    }

    /**
     * MTP 断开
     */
    public class TaskClose implements Runnable {
        BTPrinting bt = null;

        private TaskClose(BTPrinting bt) {
            this.bt = bt;
        }

        @Override
        public void run() {
            // TODO Auto-generated method stub
            bt.Close();
        }

    }

    /**
     * MTP 打印
     */
    public class TaskPrint implements Runnable {
        Pos pos = null;
        List<Map<String, String>> printMsg;

        private TaskPrint(Pos pos, List<Map<String, String>> printMap) {
            this.pos = pos;
            this.printMsg = printMap;
        }

        @Override
        public void run() {
            // TODO Auto-generated method stub

            final boolean bPrintResult = MTPPrintsSdk.PrintTicket(context, pos, printMsg,
                    1, true);
            final boolean bIsOpened = pos.GetIO().IsOpened();

            // TODO 打印成功
            if (bPrintResult) {
                if (printResultListener != null)
                    printResultListener.printSuccess();
                Log.i("MTP==", "MTP打印成功");
                isConnectAndPrint = false;
            } else {
                //todo 打印失败
                if (printResultListener != null)
                    printResultListener.printFail();
                Log.i("MTP==", "MTP打印成功");

            }
            //todo 打印的时候打印按钮不可操作

//            mActivity.runOnUiThread(new Runnable() {
//                @Override
//                public void run() {
//                    // TODO 打印成功
//                    if (bPrintResult) {
//                        es.submit(new TaskClose(mBt));
//                    } else {
//                        //todo 打印失败
//                        es.submit(new TaskClose(mBt));
//                    }
//                    //todo 打印的时候打印按钮不可操作
//                    btnPrint.setEnabled(bIsOpened);
//                    if (!bIsOpened){
//                        btnPrint.setBackgroundResource(R.drawable.cp_green_btn_bg_false);
//                    }else
//                        btnPrint.setBackgroundResource(R.drawable.cp_green_btn_bg);
//
//                }
//            });

        }
    }


    /**
     * MTP 断开打印机
     */
    private void disMTPPrinter() {
        es.submit(new TaskClose(mBt));
    }


    /**
     * ID321 连接打印机第一步
     */
    private void connectID321(String devicesAddress) {
        mDevice = BluetoothAdapter.getDefaultAdapter().getRemoteDevice(devicesAddress);
        myPrinter = PrinterInstance.getPrinterInstance(mDevice, mHandler);
//        if (mDevice.getBondState() == BluetoothDevice.BOND_NONE) {// 未绑定
            // IntentFilter boundFilter = new IntentFilter();
            // boundFilter.addAction(BluetoothDevice.ACTION_BOND_STATE_CHANGED);
            // mContext.registerReceiver(boundDeviceReceiver, boundFilter);
//            PairOrConnect(true);
//        } else {
            PairOrConnect(false);
//        }
    }

    /**
     * ID312 连接打印机第二部
     *
     * @param pair 是否配过对
     */
    private void PairOrConnect(boolean pair) {
        if (pair) {
            IntentFilter boundFilter = new IntentFilter();
            boundFilter.addAction(BluetoothDevice.ACTION_BOND_STATE_CHANGED);
            context.registerReceiver(boundDeviceReceiver, boundFilter);
            boolean success = false;
            try {

                // 开始配对 这段代码打开输入配对密码的对话框
                Method createBondMethod = BluetoothDevice.class.getMethod("createBond");
                success = (Boolean) createBondMethod.invoke(mDevice);
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (IllegalArgumentException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            }
            Log.i("ID321==", "createBond is success? : " + success);
        } else {
            new connectThread().start();
        }
    }

    /**
     * ID321连接打印机线程
     */
    private class connectThread extends Thread {
        @Override
        public void run() {
            if (myPrinter != null) {
                myPrinter.openConnection();
            }
        }
    }

    /**
     * ID321
     */
    private BroadcastReceiver boundDeviceReceiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            if (BluetoothDevice.ACTION_BOND_STATE_CHANGED.equals(action)) {
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                if (!mDevice.equals(device)) {
                    return;
                }
                switch (device.getBondState()) {
                    case BluetoothDevice.BOND_BONDING:
                        Log.i("ID321==", "bounding......");
                        break;
                    case BluetoothDevice.BOND_BONDED:
                        Log.i("ID321==", "bound success");
                        // if bound success, auto init BluetoothPrinter. open
                        // connect.
                        context.unregisterReceiver(boundDeviceReceiver);
//                        dialog.show();
                        // 配对完成开始连接
                        if (myPrinter != null) {
                            new connectThread().start();
                        }
                        break;
                    case BluetoothDevice.BOND_NONE:
                        context.unregisterReceiver(boundDeviceReceiver);
                        Log.i("ID321==", "bound cancel");
                        break;
                    default:
                        break;
                }
            }
        }
    };

    /**
     * ID321打印机 用于接受连接状态消息的 Handler
     */
    @SuppressLint("HandlerLeak")
    private Handler mHandler = new Handler() {
        @SuppressLint("ShowToast")
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case PrinterConstants.Connect.SUCCESS:
                    isConnected = true;
                    isChangePrint = false;
                    intentFilter = new IntentFilter();
                    intentFilter.addAction(BluetoothDevice.ACTION_ACL_DISCONNECTED);
                    if (isConnectAndPrint)
                        connectAndPrintHandler.sendEmptyMessage(10010);
                    isConnectAndPrint = false;
                    Log.i("ID321==", "连接成功!");
                    if (deviceConnectListener != null)
                        deviceConnectListener.connectSuccess();

                    break;
                case PrinterConstants.Connect.FAILED:


                    if (deviceConnectListener != null && !isConnected && !isChangePrinter)
                        deviceConnectListener.connectFalse();
                    isConnected = false;
                    isChangePrint = false;
                    Log.i("ID321==", "连接失败!");
                    break;
                case PrinterConstants.Connect.CLOSED:

//                    if (deviceConnectListener != null && !isConnected && !isChangePrinter)
//                        deviceConnectListener.connectFalse();
                    isConnected = false;
                    isChangePrint = false;
                    Log.i("ID321==", "连接关闭!");
                    break;
                case PrinterConstants.Connect.NODEVICE:

//                    if (deviceConnectListener != null && !isConnected && !isChangePrinter)
//                        deviceConnectListener.connectFalse();
                    isConnected = false;
                    isChangePrint = false;
                    Log.i("ID321==", "无蓝牙设备");
                    break;
                default:
                    break;
            }
        }
    };


    /**
     * ID321 打印
     */
    private void IDPrint(List<Map<String, String>> printMsg) {
        if (!blueAdapter.isEnabled()) {
            isConnected = false;
            if (blueToothEnableListner != null)
                blueToothEnableListner.enableFalse();
            return;
        }
        if (isConnected) {
            int printStatus = myPrinter.getCurrentStatus();
            if (printStatus == 0) {
                IDPrintsSdk.printBill(printMsg, myPrinter,
                        new IDPrintsSdk.printStatus() {
                            @Override
                            public void status() {
                                int printingStatus = myPrinter.getPrintingStatus(2000);
                                Log.e("=====>", myPrinter.getPrintingStatus(2000) + " ");
                                if (printingStatus == 0 || printingStatus == -2) {
                                    isConnected = true;
                                    // TODO: 2019/10/17  打印成功回调
                                    if (printResultListener != null)
                                        printResultListener.printSuccess();
                                } else {
                                    isConnected = false;
                                    if (printingStatus == -3) {
                                        // todo 打印失败 没纸提示
                                        if (printResultListener != null)
                                            printResultListener.printFail();
                                    } else {
                                        if (printResultListener != null)
                                            printResultListener.printFail();
                                    }
                                    if (printResultListener != null)
                                        printResultListener.printFail();
                                }


                            }
                        });
            } else {
                isConnected = false;
                if (printStatus == -1) {
                    // todo 打印失败：接收数据失败
                    if (printResultListener != null)
                        printResultListener.printFail();
                } else if (printStatus == -2) {
                    // todo 打印失败：The printer is out of paper
                    if (printResultListener != null)
                        printResultListener.printFail();

                } else if (printStatus == -3) {
                    // todo 打印失败：无纸 The printer paper is running out
                    if (printResultListener != null)
                        printResultListener.printFail();
                } else if (printStatus == -4) {
                    // todo 打印失败 开盖
                    if (printResultListener != null)
                        printResultListener.printFail();
                } else if (printStatus == -5) {
                    // todo 打印失败  发送数据失败
                    if (printResultListener != null)
                        printResultListener.printFail();
                }
//                // todo 执行失败回调方法
//                if (printResultListener != null)
//                    printResultListener.printFail();
            }

        } else{

            if (deviceConnectListener != null)
                deviceConnectListener.connectFalse();
        }

    }

    /**
     * 断开ID321打印机
     */
    private void disID321Printer() {
        if (myPrinter != null) {
            myPrinter.closeConnection();
            Log.i("ID321", "mPrinter:" + myPrinter);
            myPrinter = null;

        }
    }

    @SuppressLint("HandlerLeak")
    private Handler connectAndPrintHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case 10010:
                    printerToPrint(connectAndPrintMsg);
                    break;
            }
        }
    };


    /**
     * 连接并打印
     */


    Map<String, String> mapPrint;
    List<Map<String, String>> msg;

    public void connectAndPrint(Map<String, Object> printMap/*Map<String, String> mapPrint,List<Map<String, String>> msg*/) {
        disPrinter(false);
        for (String mmp : printMap.keySet()) {
            if (mmp.equals("printer") || mmp.equals("Printer")) {
                mapPrint = (Map<String, String>) printMap.get(mmp);
                System.out.print(mapPrint);
            } else {
                msg = (List<Map<String, String>>) printMap.get(mmp);
                System.out.print(msg);
            }
        }

        printerName = mapPrint.get("id");
        if (!TextUtils.equals(printerName, printerName2)) {
            isChangePrinter = true;
            printerName2 = printerName;
        } else
            isChangePrinter = false;


        isConnectAndPrint = true;
        connectAndPrintMsg = msg;

        if (!isChangePrinter&&(isMTPConnected || isConnected))
            printerToPrint(msg);
        else
            connectPrint(mapPrint);
    }


    private static  boolean isOPen(final Context context) {
        LocationManager locationManager
                = (LocationManager) context.getSystemService(Context.LOCATION_SERVICE);
        // 通过GPS卫星定位，定位级别可以精确到街（通过24颗卫星定位，在室外和空旷的地方定位准确、速度快）
        boolean gps = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
        // 通过WLAN或移动网络(3G/2G)确定的位置（也称作AGPS，辅助GPS定位。主要用于在室内或遮盖物（建筑群或茂密的深林等）密集的地方定位）
        boolean network = locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
        if (gps || network) {
            return true;
        }

        return false;
    }


    private boolean showGPSContacts() {
        if (Build.VERSION.SDK_INT >= 23) { //判断是否为android6.0系统版本，如果是，需要动态添加权限
            if (ContextCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION)
                    != PERMISSION_GRANTED) {// 没有权限，申请权限。
                return false;
            }
        }
        return true;
    }

}
