//
//  KPPrinterSeriesID.m
//  printerProje
//
//  Created by AmaniMacbook1 on 2019/10/16.
//  Copyright © 2019 AmaniMacbook1. All rights reserved.
//

#import "KPPrinterSeriesID.h"

static NSString *const kWriteCharacteristicUUID = @"49535343-8841-43F4-A8D4-ECBE34729BB3";
static NSString *const kReadCharacteristicUUID = @"49535343-1E4D-4BD9-BA61-23C647249616";
static NSString *const kServiceUUID = @"49535343-FE7D-4AE5-8FA9-9FAFD205E455";

CBPeripheral *activeDevice;
CBCharacteristic *activeWriteCharacteristic;
CBCharacteristic *activeReadCharacteristic;
int cmd;

float printOvertime = 6.0;

@interface KPPrinterSeriesID () <CBCentralManagerDelegate>

@property(strong,nonatomic)CBCentralManager *centralManager;


//@property(strong,nonatomic)CBPeripheral *printerToConnected;

@property(assign,nonatomic)BOOL ifBluetoothOpen;

@property(strong,nonatomic)NSMutableDictionary *dicPrinters;

@property(strong,nonatomic)CBPeripheral *deviceToConnect;

@property(strong,nonatomic)NSArray *arrConnectToPrint;

@property(strong,nonatomic)NSString *strSearchTime;


@end


@implementation KPPrinterSeriesID

- (instancetype)init {
    self = [super init];
    
    _centralManager = [[CBCentralManager alloc]initWithDelegate:self queue:nil];

    
    
    return self;
}

- (NSMutableDictionary *)dicPrinters {
    if (!_dicPrinters) {
        _dicPrinters = [NSMutableDictionary dictionary];
    }
    return _dicPrinters;
}


#pragma mark - ....................................printer operation

- (void)search:(NSString *)time {
    
    if (!self.ifBluetoothOpen) {
        if (self.searchResult_block) {
            self.searchResult_block(@"BluetoothIsClosed");
        }
    }
    
    if (time.length > 0) {
        self.strSearchTime = time;
    }else{
        self.strSearchTime = @"5";
    }
    
    NSLog(@"%s",__func__);
    
    //清空上次打印机搜索结果
    [self.dicPrinters removeAllObjects];
    
    if (activeDevice.state==CBPeripheralStateConnected) {
        //需要先断开打印机
        //
        [self disconectPrinter:nil];

    }else{
        [self.centralManager scanForPeripheralsWithServices:nil options:nil];
        
        [NSTimer scheduledTimerWithTimeInterval:self.strSearchTime.intValue target:self selector:@selector(stopScanPeripheral) userInfo:nil repeats:NO];
    }
    
}

- (void)connectPrinter:(NSDictionary *)aDic {
    NSLog(@"%s,%@",__func__,aDic);
    NSLog(@"所有打印机：%@",self.dicPrinters);
    
    //需要先断开打印机
    //
//    [self disconectPrinter:nil];
    
    
    NSString *strKey = aDic[@"id"];
    CBPeripheral * device = self.dicPrinters[strKey];

    
    if (device) {
        
        if(device.state==CBPeripheralStateConnected)
        {
            //如果点击的是当前已连接的设备，则不进行任何操作
            //NSLog(@"buttonStartDisconnect\n");
            //[bluetoothRadio startDisconnectDevice:device];
            //        [centralManager cancelPeripheralConnection:device];
            
            
        }
        else
        {
            //先断开当前连接
            if (activeDevice) {
                _deviceToConnect = device;
                [self disconectPrinter:nil];
            }else{
                
                //        [self.centralManager stopScan];
                NSLog(@"stop scan");
                
                activeDevice = device;
                //[self.centralManager stopScan];
                //NSLog(@"buttonStartConnect\n");
                //[bluetoothRadio startConnectDevice:device timeout:DEFAULT_CONNECT_TIMEOUT];
                [_centralManager connectPeripheral:device options:@{CBConnectPeripheralOptionNotifyOnConnectionKey : @YES}];


//                activeDevice = self.printerToConnected;
//                [_centralManager connectPeripheral:self.printerToConnected options:@{CBConnectPeripheralOptionNotifyOnConnectionKey : @YES}];
                NSLog(@"连接中...");
            }
            
        }

    }
}

- (void)disconectPrinter:(NSDictionary *)aDic {
    NSLog(@"%s",__func__);
    

    if (activeDevice.state==CBPeripheralStateConnected) {
        [_centralManager cancelPeripheralConnection:activeDevice];
    }

    
}

- (void)print:(NSArray *)aArr {
    NSLog(@"%s",__func__);
    
    NSLog(@"开始打印");
    
    if (!activeDevice) {
        NSLog(@"请先连接打印机");
        return;
    }
    
    for (int i = 0; i < aArr.count; i++) {
        NSDictionary *dic = aArr[i];
        if ([dic[@"LayoutStyle"] isEqualToString:@"enter"]) {
            [Print printTxt:@"\n"];
        }else if ([dic[@"LayoutStyle"] isEqualToString:@"signlineSeperate"]) {
            [Print printAlignCenter];
            if ([activeDevice.name hasPrefix:@"ID"]) {
                [Print printTxt:@"______________    ______________\n"];
            }else{
                [Print printTxt:@"———————  ———————\n"];
            }
        }else{
            if ([dic[@"LayoutStyle"] isEqualToString:@"left"]) {
                [Print printAlignLeft];
            }else if ([dic[@"LayoutStyle"] isEqualToString:@"center"]) {
                [Print printAlignCenter];
            }else if ([dic[@"LayoutStyle"] isEqualToString:@"right"]) {
                [Print printAlignRight];
            }
            [Print printTxt:[NSString stringWithFormat:@"%@\n",dic[@"content"]]];
        }
    }
    
    [Print printTxt:@"\n"];
    [Print printTxt:@"\n"];
    [Print printTxt:@"\n"];

    [Print getPrinterState];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(printOvertime * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        dispatch_async(dispatch_get_main_queue(), ^{
            [Print getPrinterStatus:2];
        });
    });
    
    self.arrConnectToPrint = nil;
}

- (void)bluetoothIsOpen {
    NSLog(@"%s",__func__);
    
    
}

- (void)connectAndPrint:(NSDictionary *)aDic {
    NSLog(@"connectAndPrint:%@",aDic);
    NSDictionary *dicPrinter = aDic[@"printer"];
    NSString *strKey = dicPrinter[@"id"];
    CBPeripheral * device = self.dicPrinters[strKey];
    
    if ([device isEqual:activeDevice]) {
        //直接打印
        [self print:aDic[@"printConent"]];
    }else{
        self.arrConnectToPrint = aDic[@"printConent"];
        [self connectPrinter:dicPrinter];

        
        
        
        
//        //如果当前连接的打印机不是要进行打印的打印机
//        if (activeDevice) {
//            //如果连接了打印机(先断开当前连接，再连接，再打印)
//            [self connectPrinter:dicPrinter];
//
//
//        }else{
//            //如果当前为未连接状态
//            //连接(连接成功之后进行打印)
//            [self connectPrinter:dicPrinter];
//
//        }
    }
    
    
    
    
    
}

- (void)printIt {
    [self print:self.arrConnectToPrint];
}

#pragma mark -

- (void) stopScanPeripheral
{
    [self.centralManager stopScan];
    NSLog(@"stop scan");
    if (self.searchResult_block) {
        self.searchResult_block(@"true");
    }
}



#pragma mark - CBPeripheralDelegate
- (void)peripheral:(CBPeripheral *)peripheral didWriteValueForCharacteristic:error
{
    NSLog(@"%s",__FUNCTION__);
    
    if (error==nil)
    {
        NSLog(@"Write edata failed!");
        return;
    }
    NSLog(@"Write edata success!");
}
- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error
{
    NSLog(@"%s",__FUNCTION__);
    
    if (error==nil)
    {
        //在这个方法中我们要查找到我们需要的服务  然后调用discoverCharacteristics方法查找我们需要的特性
        //该discoverCharacteristics方法调用完后会调用代理CBPeripheralDelegate的
        //- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error
        for (CBService *service in peripheral.services)
        {
            if ([service.UUID isEqual:[CBUUID UUIDWithString:kServiceUUID]])
            {
                //[peripheral discoverCharacteristics:@[[CBUUID UUIDWithString:kCharacteristicUUID]] forService:service];
                [peripheral discoverCharacteristics:nil forService:service];
            }
        }
    }
}

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error{
    NSLog(@"%s",__FUNCTION__);
    
    if (error==nil) {
        //在这个方法中我们要找到我们所需的服务的特性 然后调用setNotifyValue方法告知我们要监测这个服务特性的状态变化
        //当setNotifyValue方法调用后调用代理CBPeripheralDelegate的- (void)peripheral:(CBPeripheral *)peripheral didUpdateNotificationStateForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
        for (CBCharacteristic *characteristic in service.characteristics)
        {
            if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:kWriteCharacteristicUUID]])
            {
                [peripheral setNotifyValue:YES forCharacteristic:characteristic];
                activeWriteCharacteristic = characteristic;
            }
            else
                if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:kReadCharacteristicUUID]])
                {
                    [peripheral setNotifyValue:YES forCharacteristic:characteristic];
                    activeReadCharacteristic = characteristic;
                }
            activeDevice = peripheral;
        }
    }
}

- (void)peripheral:(CBPeripheral *)peripheral didUpdateNotificationStateForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    NSLog(@"%s",__FUNCTION__);
    
    NSLog(@"enter didUpdateNotificationStateForCharacteristic!");
    if (error==nil)
    {
        //调用下面的方法后 会调用到代理的- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
        [peripheral readValueForCharacteristic:characteristic];
    }
}

//打印完成回调
- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error
{
    NSLog(@"%s:%@",__FUNCTION__,error);
    //    if (!error) {
    //        //打印成功
    ////        [HBProgressHUD showText:LS(@"打印成功")];
    //
    //        if (self.printResult_block) {
    //            self.printResult_block(cFlutterResultSuccess);
    //        }
    //        [self dismissViewControllerAnimated:YES completion:nil];
    //    }
    ////    else{
    ////        //打印失败
    ////        if (self.printResult_block) {
    ////            self.printResult_block(cFlutterResultFailure);
    ////        }
    ////        [HBProgressHUD showText:LS(@"打印失败")];
    ////    }
    
    NSLog(@"enter didUpdateValueForCharacteristic!");
    NSData *data = characteristic.value;
    NSLog(@"read data=%@!",data);
    
    const Byte *res =[data bytes];
    
    //返回的数据 0x37,0x22,0x30,0x30,0x30,0x30,0x00 说明打印完毕，否则还在打印中...
    NSInteger dlen=[data length];
    for (uint i=0;i<[data length];i++)
    {
        uint state=res[i];
        
        if (state == 50 || state == 82) {
            NSLog(@"打印失败");
            if (self.printResult_block) {
                self.printResult_block(@"false");
            }
            if (self.connectAndPrintResult_block) {
                self.connectAndPrintResult_block(@"false");
            }
            continue;
        }
        
        if(state!=55)continue;
        if(i+1>=dlen) continue;
        
        if(res[i+1]!=34)continue;
        if(i+2>=dlen) continue;
        
        if(res[i+2]!=48)continue;
        if(i+3>=dlen) continue;
        
        if(res[i+3]!=48)continue;
        if(i+4>=dlen) continue;
        
        if(res[i+4]!=48)continue;
        if(i+5>=dlen) continue;
        
        if(res[i+5]!=48)continue;
        if(i+6>=dlen) continue;
        
        if(res[i+6]!=0)continue;
        //说明打印完毕
        //        statusText.text=@"打印完毕！！";
        NSLog(@"打印成功");
        //打印成功
        if (self.printResult_block) {
            self.printResult_block(@"true");
        }
        if (self.connectAndPrintResult_block) {
            self.connectAndPrintResult_block(@"true");
        }
        
    }
    
    
    char hex_resp[[data length]*2+1];
    char temp[3];
    memset(hex_resp, 0, [data length]*2+1);
    memset(temp,0,3);
    
    switch (cmd)
    {
            
        case DLE_EOT_1:
            
            if (res[0] & 0x08) NSLog(@"打印失败：offline");
            //            else  statusText.text=@"normal";
            break;
            
        case DLE_EOT_3:
            if (res[0] & 0x68) NSLog(@"打印失败：print error");
            //            else  statusText.text=@"normal";
            break;
        case DLE_EOT_4:
            if (res[0] & 0x60) NSLog(@"打印失败：out of paper");
            //            else  statusText.text=@"normal";
            break;
        default:
            
            for (int i=0;i<[data length];i++)
            {
                
                sprintf(temp,"%02x",res[i]);
                strcat(hex_resp,temp);
            }
            NSLog(@"打印无错误");
            //            statusText.text=[NSString stringWithUTF8String:hex_resp];
            break;
    }
    cmd = 0;
    
    
}


#pragma mark - Bluetooth.delegate

-(void)centralManagerDidUpdateState:(CBCentralManager *)central {
    NSLog(@"%s",__FUNCTION__);
    
    switch (central.state) {
        case CBCentralManagerStatePoweredOn:
            NSLog(@"蓝牙已打开,请扫描外设");
            self.ifBluetoothOpen = YES;
//            [self search];
            break;
            
        case CBCentralManagerStatePoweredOff:
            NSLog(@"蓝牙关闭...");
            self.ifBluetoothOpen = NO;
            [self disconectPrinter:nil];
            break;
            
        default:
            break;
    }
}

//搜索到硬件，带着硬件信息回调此结果
- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI{
//    NSLog(@"%s",__FUNCTION__);
    
//    NSLog(@"searched:%@",[NSString stringWithFormat:@"%@  (RSSI:%d,id:%@)", peripheral.name, RSSI,peripheral.identifier]);
    
//    if (!peripheral.name) {
//        return;
//    }

    //id231
//    if ([peripheral.identifier.UUIDString isEqualToString:@"10EE51A4-C47C-4751-B3D7-66CA3FD20730"]) {
//        self.printerToConnected = peripheral;
//    }
    
    //mtp
//    if ([peripheral.identifier.UUIDString isEqualToString:@"83F70F1C-81A8-4472-A94A-6388AE957BC0"]) {
//        self.printerToConnected = peripheral;
//    }

    //屏蔽打印机i以外的设备
    if (![peripheral.name hasPrefix:@"ID"] && ![peripheral.name hasPrefix:@"MTP"]) {
        return;
    }

    [self.dicPrinters setValue:peripheral forKey:peripheral.identifier.UUIDString];
    
//    if (self.searchResult_block) {
//        NSLog(@"searched:%@",[NSString stringWithFormat:@"%@  (RSSI:%d,id:%@)", peripheral.name, RSSI,peripheral.identifier]);
//        self.searchResult_block(@{@"name":((peripheral.name&&![peripheral.name isEqual:[NSNull null]])?peripheral.name:@"--"),@"id":peripheral.identifier.UUIDString});
//    }
    
    if (self.eventSink_block) {
        self.eventSink_block(@{@"name":((peripheral.name&&![peripheral.name isEqual:[NSNull null]])?peripheral.name:@"--"),@"id":peripheral.identifier.UUIDString});
    }
    
    
    if (peripheral && peripheral.name && ![peripheral.name isEqual:[NSNull null]])
    {
        NSLog(@"foundDevice. name[%s],RSSI[%d]\n,id:%@",peripheral.name.UTF8String,peripheral.RSSI.intValue,peripheral.identifier);
        //if ( [peripheral.name isEqualToString:@"T9 BT Printer"] )
        {
            //activeDevice = peripheral;
            //发现设备后即可连接该设备 调用完该方法后会调用代理CBCentralManagerDelegate的- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral表示连接上了设别
            //如果不能连接会调用 - (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
            //[centralManager connectPeripheral:peripheral options:@{CBConnectPeripheralOptionNotifyOnConnectionKey : YES}];
            //            if ([peripheral.name isEqualToString:@"ID321 Printer"]) {
        }
    }
}

- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    NSLog(@"%s",__FUNCTION__);
    NSLog(@"has connected");
    //连接成功
    if (self.connectResult_block) {
        self.connectResult_block(@"true");
    }
    
    _deviceToConnect = nil;
    //[mutableData setLength:0];
    activeDevice.delegate = self;
    //此时设备已经连接上了  你要做的就是找到该设备上的指定服务 调用完该方法后会调用代理CBPeripheralDelegate（现在开始调用另一个代理的方法了）的
    //- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error
    [activeDevice discoverServices:@[[CBUUID UUIDWithString:kServiceUUID]]];
    if (self.arrConnectToPrint.count > 0) {
        [self performSelector:@selector(printIt) withObject:nil afterDelay:1.0];
    }
    
}
- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error
{
    NSLog(@"%s",__FUNCTION__);
    
    NSLog(@"Peripheral Disconnected");
    activeDevice = nil;
    //    [self alertMessage:@"连接断开！"];
    NSLog(@"连接已断开");
    
    if (self.disconnectResult_block) {
        self.disconnectResult_block(@"true");
    }
    
    if (_deviceToConnect) {
        activeDevice = _deviceToConnect;
        //[self.centralManager stopScan];
        //NSLog(@"buttonStartConnect\n");
        //[bluetoothRadio startConnectDevice:device timeout:DEFAULT_CONNECT_TIMEOUT];
        [_centralManager connectPeripheral:activeDevice options:@{CBConnectPeripheralOptionNotifyOnConnectionKey : @YES}];
        NSLog(@"断开后连接中。。。");
    }
    
    [self search:self.strSearchTime];

    
}




- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error{
    NSLog(@"%s",__FUNCTION__);
    
    //此时连接发生错误
    NSLog(@"连接失败");
    
    self.arrConnectToPrint = nil;
}


#pragma mark - ---

- (void)printContent {
    
    
}



@end
