#import "KpprinterPlugin.h"
#import "KPPrinterSeriesID.h"

@interface KpprinterPlugin () <FlutterStreamHandler>

@property (strong ,nonatomic) KPPrinterSeriesID *printerSeariesID;
@property (strong ,nonatomic) FlutterEventChannel *eventChannel;

@end

@implementation KpprinterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"kpprinter"
            binaryMessenger:[registrar messenger]];
  KpprinterPlugin* instance = [[KpprinterPlugin alloc] init];
    instance.printerSeariesID = [[KPPrinterSeriesID alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
    
    FlutterEventChannel *eventChannel = [FlutterEventChannel eventChannelWithName:@"printer_printerSearched" binaryMessenger:[registrar messenger]];
    [eventChannel setStreamHandler:instance];



}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSLog(@"call.method:%@",call.method);
    NSString *strMethod = [call.method stringByAppendingString:@":"];
    SEL selector = NSSelectorFromString(strMethod);
    
    if ([self respondsToSelector:selector]) {
        [self performSelector:selector withObject:@{@"call":call,@"result":result}];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (FlutterError* _Nullable)onListenWithArguments:(id _Nullable)arguments
                                       eventSink:(FlutterEventSink)events {
    NSLog(@"flutter开始进行监听，并在此方法传入 原生主动传值给flutter的桥梁 event");
    self.printerSeariesID.eventSink_block = events;
    return nil;
}

- (FlutterError* _Nullable)onCancelWithArguments:(id _Nullable)arguments {
    NSLog(@"%s:%@",__func__,arguments);
    return nil;
}

#pragma mark - 搜索
- (void)printer_search:(id)aObject{
    __weak typeof(self) weakself = self;

    NSDictionary *dic = (NSDictionary *)aObject;
    FlutterResult fluresult = dic[@"result"];
    FlutterMethodCall *call = dic[@"call"];

    [self.printerSeariesID search:call.arguments];
    self.printerSeariesID.searchResult_block = ^(NSString *aStrResult) {
        [weakself showText:[NSString stringWithFormat:@"搜索结果：%@",aStrResult]];
        fluresult(aStrResult);
    };

}

#pragma mark - 连接打印机
- (void)printer_connectAPrinter:(id)aObject{
    __weak typeof(self) weakself = self;

    NSDictionary *dic = (NSDictionary *)aObject;
    FlutterResult fluresult = dic[@"result"];
    FlutterMethodCall *call = dic[@"call"];

    [self.printerSeariesID connectPrinter:call.arguments];
    self.printerSeariesID.connectResult_block = ^(NSString *aStrResult) {
        [weakself showText:[NSString stringWithFormat:@"连接结果：%@",aStrResult]];
        fluresult(aStrResult);
    };

}

#pragma mark - 断开打印机
- (void)printer_disconnectAPrinter:(id)aObject{
    __weak typeof(self) weakself = self;

    NSDictionary *dic = (NSDictionary *)aObject;
    FlutterResult fluresult = dic[@"result"];
    FlutterMethodCall *call = dic[@"call"];

    [self.printerSeariesID disconectPrinter:call.arguments];
    self.printerSeariesID.disconnectResult_block = ^(NSString *aStrResult) {
        [weakself showText:[NSString stringWithFormat:@"断开结果：%@",aStrResult]];
        fluresult(aStrResult);

    };

}

#pragma mark - 打印
- (void)printer_print:(id)aObject{
    __weak typeof(self) weakself = self;

    NSDictionary *dic = (NSDictionary *)aObject;
    FlutterResult fluresult = dic[@"result"];
    FlutterMethodCall *call = dic[@"call"];

    [self.printerSeariesID print:call.arguments];
    self.printerSeariesID.printResult_block = ^(NSString *aStrResult) {
        [weakself showText:[NSString stringWithFormat:@"打印结果：%@",aStrResult]];
        fluresult(aStrResult);
    };

}

#pragma mark - 判断蓝牙是否打开
- (void)printer_bluetoothIsOpened:(id)aObject{
    
    NSDictionary *dic = (NSDictionary *)aObject;
    FlutterResult fluresult = dic[@"result"];

    [self.printerSeariesID bluetoothIsOpen];
    self.printerSeariesID.bluetoothIsOpenResult_block = ^(NSString *aStrResult) {
        fluresult(aStrResult);
    };

}

#pragma mark - 连接打印机
- (void)printer_connectAndPrint:(id)aObject{
    __weak typeof(self) weakself = self;
    
    NSDictionary *dic = (NSDictionary *)aObject;
    FlutterResult fluresult = dic[@"result"];
    FlutterMethodCall *call = dic[@"call"];
    
    [self.printerSeariesID connectAndPrint:call.arguments];
    self.printerSeariesID.connectAndPrintResult_block = ^(NSString *aStrResult) {
        [weakself showText:[NSString stringWithFormat:@"连接并打印结果：%@",aStrResult]];
        fluresult(aStrResult);
    };
    
}

#pragma mark -

- (void)showText:(NSString *)aText {
    return;
    UIAlertView *alertView = [[UIAlertView alloc]initWithTitle:@"提示" message:aText delegate:self cancelButtonTitle:@"好的" otherButtonTitles:nil];
    alertView.alertViewStyle = UIAlertViewStyleDefault;
    [alertView show];
    
}



@end
